<?php


namespace app\cbcrm\controller\api;

/**
 * 支付通知回调
 * */

use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\goods\GoodsModel;
use app\cbcrm\model\goods\OrderDetailModel;
use app\cbcrm\model\goods\OrderModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\model\member\RechargeModel;
use app\cbcrm\model\trade\TrademarkLocalModel;
use app\cbcrm\model\trade\TrademarkOrderModel;
use app\cbcrm\model\trade\TrademarkOrderCostModel;
use app\cbcrm\model\trademark\ServiceOrderModel;
use app\cbcrm\services\SmsServices;
use app\cbcrm\services\TrademarkService;
use EasyWeChat\Factory;
use EasyWeChat\Kernel\Exceptions\Exception;
use think\facade\Db;
use think\facade\Log;

class Notify extends Base
{

    /*
     * 用户充值支付回调
     * */
    public function recharge()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message, $fail) use ($app) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $rechargeOrder = (new RechargeModel())->where('order_no', $message['out_trade_no'])->find();

                if (!$rechargeOrder || $rechargeOrder->pay_status == 1) { // 如果订单不存在 或者 订单已经支付过了
                    return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($rechargeOrder->pay_money) * 100) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态

                    // 充值入款
                    $rechargeOrder->pay_status = 1;
                    $rechargeOrder->transaction_id = $message['transaction_id'];
                    $rechargeOrder->pay_time = date('Y-m-d H:i:s');

                    $rechargeOrder->save();

                    // 插入充值记录
                    Db::table('cb_crm_user_balance_recharge_log')->insert([
                        'wid' => $rechargeOrder->wid,
                        'user_id' => $rechargeOrder->user_id,
                        'scene' => 1, // 变动场景:1=用户充值,2=用户消费,3=管理员操作,4=订单退款
                        'money' => $rechargeOrder->actual_money,
                        'create_time' => $rechargeOrder->pay_time,
                        'update_time' => $rechargeOrder->pay_time,
                        'remark' => '账户余额充值'
                    ]);

                    // 变更用户余额
                    $res = (new MemberModel())->where('id', $rechargeOrder->user_id)->inc('balance', $rechargeOrder->actual_money)->update();

                    if ($res) {
                        return true;
                    } else {
                        return $fail('充值异常，请稍后再通知我');
                    }
                }
                return $fail('充值异常，请稍后再通知我');

            });
            $response->send();
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * 商品订单支付回调
     * */
    public function order()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message,$fail) use ($app,$wid) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $order = (new OrderModel())->where('order_no',$message['out_trade_no'])->find();
                if (!$order || $order->pay_status == 1) { // 如果订单不存在 或者 订单已经支付过了
                    return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($order->pay_price) * 100) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态

                    // 修改订单支付状态
                    $order->pay_status = 1;
                    $order->pay_time = date('Y-m-d H:i:s');
                    $order->transaction_id = $message['transaction_id'];


                    $orderDetailList = (new OrderDetailModel())->where('order_id',$order->id)->select();
                    foreach ($orderDetailList as $key=>$item) {
                        $goodsInfo = (new GoodsModel())->where('id',$item->goods_id)->find();
                        // 付款减库存
                        if ($item->deduct_stock_type == 2) {
                            $goodsInfo->stock_total = $goodsInfo->stock_total - $item->total_num;
                            Db::table('cb_crm_goods_spec')->where('goods_id', $item->goods_id)->where('id', $item->spec_id)->dec('stock_num', $item->total_num)->update();
                        }
                        // 虚拟物品，默认已发货
                        if ($goodsInfo->type == 0) {
                            $order->delivery_status = 1;
                            $order->delivery_time = date('Y-m-d H:i:s');
                        }
                        // 增加销量
                        $goodsInfo->sales_actual = $goodsInfo->sales_actual + $item->total_num;
                        $goodsInfo->sales_initial = $goodsInfo->sales_initial + $item->total_num;
                        $goodsInfo->save();
                    }
                    // 订单确认收货后再进行 赠送积分、增加用户实际消费金额、改变会员等级等操作
                    $res = $order->save();

                    if ($res) {
                        // 发送短信通知
                        $smsServices = app()->make(SmsServices::class);
                        $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
                        $smsServices->send($mobile,['type'=>'商品','order_no'=>$message['out_trade_no']],'后台新订单',$wid);
                        return true;
                    } else {
                        return $fail('订单异常，请稍后再通知我');
                    }
                }
                return $fail('订单异常，请稍后再通知我');

            });
            $response->send(); // return $response
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * logo订单支付回调
     * */
    public function logo_order()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message, $fail) use ($app,$wid) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $order = (new \app\cbcrm\model\logo\OrderModel())->where('order_no',$message['out_trade_no'])->find();
                if (!$order || $order->pay_status == 1) { // 如果订单不存在 或者 订单已经支付过了
                    return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($order->pay_price) * 100) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态
                    // 修改订单支付状态
                    $order->pay_status = 1;
                    $order->pay_time = date('Y-m-d H:i:s');
                    $res = $order->save();
                    if ($res) {
                        // 发送短信通知
                        $smsServices = app()->make(SmsServices::class);
                        $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
                        $smsServices->send($mobile,['type'=>'LOGO设计','order_no'=>$message['out_trade_no']],'后台新订单',$wid);
                        return true;
                    } else {
                        return $fail('订单异常，请稍后再通知我');
                    }
                }
                return $fail('订单异常，请稍后再通知我');

            });
            $response->send(); // return $response
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * 商标注册订单支付回调
     * */
    public function trademerk_order()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message, $fail) use ($app,$wid) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $orderList = (new \app\cbcrm\model\trademark\OrderModel())->where('order_no',$message['out_trade_no'])->select();
                $amount = 0.00;
                foreach ($orderList as $order) {
                    if (!$order || $order->pay_status == 1) { // 如果订单不存在 或者 订单已经支付过了
                        return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                    }
                    $amount+=$order->amount;
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($amount * 100)) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态

                    foreach ($orderList as $order) {// 修改订单支付状态
                        $order->pay_status = 1;
                        $order->pay_time = date('Y-m-d H:i:s');
                        $res = $order->save();
                        // 修改自有商标交易库

                    }
                    if ($res) {
                        // 发送短信通知
                        $smsServices = app()->make(SmsServices::class);
                        $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
                        $smsServices->send($mobile,['type'=>'商标注册','order_no'=>$message['out_trade_no']],'后台新订单',$wid);
                        return true;
                    } else {
                        return $fail('订单异常，请稍后再通知我');
                    }
                }
                return $fail('订单异常，请稍后再通知我');

            });
            $response->send(); // return $response
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * 商标服务订单支付回调
     * */
    public function tm_service_order()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message, $fail) use ($app,$wid) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $order = (new ServiceOrderModel())->where('order_no',$message['out_trade_no'])->find();
                if (!$order || $order->status != 0) { // 如果订单不存在 或者 订单已经支付过了
                    return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($order->amount) * 100) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态
                    // 修改订单支付状态
                    $order->status = 1;
                    $order->pay_time = date('Y-m-d H:i:s');
                    $res = $order->save();
                    if ($res) {
                        // 发送短信通知
                        $smsServices = app()->make(SmsServices::class);
                        $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
                        $smsServices->send($mobile,['type'=>'商标服务','order_no'=>$message['out_trade_no']],'后台新订单',$wid);
                        return true;
                    } else {
                        return $fail('订单异常，请稍后再通知我');
                    }
                }
                return $fail('订单异常，请稍后再通知我');

            });
            $response->send(); // return $response
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * 版权登记订单支付回调
     * */
    public function copyright_order()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message, $fail) use ($app,$wid) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $order = (new \app\cbcrm\model\copyright\OrderModel())->where('order_no',$message['out_trade_no'])->find();
                if (!$order || $order->pay_status != 0) { // 如果订单不存在 或者 订单已经支付过了
                    return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($order->amount) * 100) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态
                    // 修改订单支付状态
                    $order->pay_status = 1;
                    $order->pay_time = date('Y-m-d H:i:s');
                    $res = $order->save();
                    if ($res) {
                        // 发送短信通知
                        $smsServices = app()->make(SmsServices::class);
                        $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
                        $smsServices->send($mobile,['type'=>'版权登记','order_no'=>$message['out_trade_no']],'后台新订单',$wid);
                        return true;
                    } else {
                        return $fail('订单异常，请稍后再通知我');
                    }
                }
                return $fail('订单异常，请稍后再通知我');

            });
            $response->send(); // return $response
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * 商标交易支付回调
     * */
    public function tm_trade_order()
    {
        $wid = $this->request->param('wid');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        $platform = $this->request->param('platform');

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        //微信小程序 必要配置
        if ($platform == 'mpWechat') {
            $payConfig['app_id'] = $config['miniapp_id'];
            $payConfig['secret'] = $config['miniapp_secret'];
        }
        $app = Factory::payment($payConfig);
        try {
            $response = $app->handlePaidNotify(function ($message, $fail) use ($app,$wid) {

                // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
                $order = (new TrademarkOrderCostModel())->where('order_no',$message['out_trade_no'])->find();
                if (!$order || $order->pay_status != 0) { // 如果订单不存在 或者 订单已经支付过了
                    return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
                }

                // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
                $result = $app->order->queryByOutTradeNumber($message['out_trade_no']);
                if ($result['trade_state'] != 'SUCCESS' || $result['cash_fee'] != ($order->price) * 100) {
                    return $fail('支付异常，请稍后再通知我');
                }

                if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态
                    // 修改订单支付状态
                    $order->pay_status = 1;
                    $order->pay_type = 'wechat';
                    $order->pay_time = date('Y-m-d H:i:s');
                    $res = $order->save();
                    // 修改商标订单状态
                    (new TrademarkOrderModel())->where('id',$order->order_id)->update([
                        'pay_status'=>(int)$order->type === 1?1:2,
                        'pay_time'=>date('Y-m-d H:i:s'),
                        'status'=>(int)$order->type === 1?2:4
                    ]);
                    // 修改自有商标交易库的状态
                    $tradeRow = (new TrademarkOrderModel())->where('id',$order->order_id)->find();
                    if ($tradeRow->type==1 && $tradeRow->local_id) {
                        (new TrademarkLocalModel())->where('id',$tradeRow->local_id)->update([
                            'admin_status'=>0,
                        ]);
                    }
                    $trademarkServices = app()->make(TrademarkService::class);

                    $api_key = (new SettingModel())->getConfigValue('trade', 'trade_key', $wid);
                    if ($order->type == 1) {
                        // 创建云交易库订单
                        $trademarkServices->createCloudOrder($tradeRow->id,$api_key,$tradeRow);
                    } elseif ($order->type == 2) {
                        // 更新云交易库订单
                        $trademarkServices->updateCloudOrder($api_key, [
                            'cloud_id'=>$tradeRow['cloud_id'],
                            'unique_id' => $tradeRow['unique_id'],
                            'pay_status' => (int)$order->type === 1 ? 1 : 2,
                            'status' => (int)$order->type === 1 ? 2 : 4
                        ]);
                    }

                    if ($res) {
                        // 发送短信通知
                        $smsServices = app()->make(SmsServices::class);
                        $mobile = (new SettingModel())->getConfigValue('sms','acceptPhone', $wid);
                        $smsServices->send($mobile,['type'=>'商标交易','order_no'=>$message['out_trade_no']],'后台新订单',$wid);
                        return true;
                    } else {
                        return $fail('订单异常，请稍后再通知我');
                    }
                }
                return $fail('订单异常，请稍后再通知我');

            });
            $response->send(); // return $response
        } catch (Exception $e) {
            Log::error($e->getMessage());
            $this->error($e->getMessage());
        }
    }

    /*
     * 查询商品订单支付状态
     * */
    public function getOrderStatus(){
        $wid = $this->request->param('wid');
        $order_no = $this->request->param('order_no');

        $config = (new SettingModel())->getConfigGroup('wechat', $wid);

        //微信公众号 必要配置
        $payConfig = [
            'app_id' => $config['app_id'],
            'mch_id' => $config['mch_id'],
            'key' => $config['api_key'],   // API v2 密钥 (注意: 是v2密钥 是v2密钥 是v2密钥)
        ];
        $app = Factory::payment($payConfig);

        // 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付
        $result = $app->order->queryByOutTradeNumber($order_no);
        if ($result['trade_state'] != 'SUCCESS') {
            $this->success('未支付',false)
;        }else{
            $this->success('已支付',true);
        }
    }
}
