<?php


namespace app\cbcrm\controller\crm;


use app\cbcrm\controller\BaseUser;
use think\facade\Db;
use think\facade\Filesystem;
use think\facade\Log;

class Attachment extends BaseUser
{
    protected $imageExt = ['jpg','jpeg','png','gif','webp'];
    /*
     * 附件列表
     * */
    public function index()
    {
        $group_id = request()->get('group_id', 0);
        $filename = request()->get('filename','');
        $where = [];
        if ($group_id != 0) {
            $all = Db::table('cb_crm_attachment_group')->where($where)->where('wid', $this->wid)->select();
            $group_ids = getChildrenIds($group_id, $all);
            // 包含自身
            array_push($group_ids,$group_id);
            $where[] = ['group_id', 'IN', $group_ids];
        }
        if ($filename) {
            $where[] = ['filename', 'LIKE', '%'.$filename.'%'];
        }
        $result = Db::table('cb_crm_attachment')->where($where)->where('wid', $this->wid)->order('id', 'desc')->paginate(15);
        $items = $result->items();
        foreach ($items as $key => $item) {
            if (in_array(strtolower($item['ext']),$this->imageExt)) {
                $items[$key]['preview_url'] = cdnurl($item['url'], true);
            }else{
                $items[$key]['preview_url'] = build_base64_image();
            }
        }
        $data = [
            'current_page' => $result->currentPage(),
            'last_page' => $result->lastPage(),
            'data' => $items,
            'per_page' => 15,
            'total' => $result->total(),
        ];

        return success('返回成功', $data);
    }

    /*
     * 添加分组
     * */
    public function group_add()
    {
        $params = request()->post('form');
        $params['wid'] = $this->wid;
        $params['create_time'] = date('Y-m-d H:i:s');
        $params['update_time'] = date('Y-m-d H:i:s');
        $res = Db::table('cb_crm_attachment_group')->insert($params);
        return success('创建成功', $res);
    }

    /*
     * 移动分组
     * */
    public function move_group()
    {
        $fileIds = request()->post('fileIds/a');
        $group_id = request()->post('group_id');

        $res = Db::table('cb_crm_attachment')->where('wid', $this->wid)->where('id', 'IN', $fileIds)->update([
            'group_id' => $group_id,
            'update_time' => date('Y-m-d H:i:s')
        ]);
        return success('移动成功', $res);
    }

    /*
     * 附件上传
     * */
    public function upload_image()
    {
        $file = request()->file('file');
        $group_id = request()->post('group_id');
        $cdnurl  = request()->post('cdnurl',1);
        if (!$file) return error('未检测到上传资源');
        $domain = $this->req->domain();
        $ext = $file->extension(); //获取后缀
        $minType = $file->getMime(); //获取附件类型
        $fileName = $file->getOriginalName(); //获取上传名
        $fileSize = $file->getSize();
        $time = date('Y-m-d');
        $res = Filesystem::putFile('upload/cbcrm/' . $time, $file, 'uniqid');
        if ($cdnurl) {
            $url = $domain . '/storage/' . $res;
        }else{
            $url = $res;
        }

        $data = [
            'wid' => $this->wid,
            'group_id' => $group_id,
            'url' => $res,
            'imagetype' => $minType,
            'filename' => $fileName,
            'filesize' => $fileSize,
            'ext' => $ext,
            'create_time' => date('Y-m-d H:i:s'),
            'update_time' => date('Y-m-d H:i:s')
        ];
        $result = Db::table('cb_crm_attachment')->insert($data);
        if ($result) {
            return success('返回成功', ['url' => $url]);
        } else {
            return error('上传失败');
        }

    }

    public function edit()
    {
        $id = request()->post('id',0,'intval');
        $params = request()->post('form');
        $row = Db::table('cb_crm_attachment')->where('wid',$this->wid)->where('id',$id)->value('id');
        if (!$row) {
            return error('附件不存在');
        }
        $result = Db::table('cb_crm_attachment')->where('wid',$this->wid)->where('id',$id)->update($params);
        if ($result) {
            return success('编辑成功');
        } else {
            return error('编辑失败');
        }
    }

    public function del()
    {
        $ids = request()->delete('fileIds/a');
        $result = Db::table('cb_crm_attachment')->where('wid',$this->wid)->where('id','IN',$ids)->delete();
        if ($result) {
            return success('删除成功');
        } else {
            return error('删除失败');
        }
    }

    public function group_list()
    {
        $list = Db::table('cb_crm_attachment_group')->where('wid', $this->wid)->order('sort desc,id desc')->select();
        return success('返回成功', getTreeData($list, 0));
    }

    public function group_edit()
    {
        $id = request()->post('id',0,'intval');
        $params = request()->post('form');
        $row = Db::table('cb_crm_attachment_group')->where('wid',$this->wid)->where('id',$id)->value('id');
        if (!$row) {
            return error('附件分组不存在');
        }
        $result = Db::table('cb_crm_attachment_group')->where('wid',$this->wid)->where('id',$id)->update($params);
        if ($result) {
            return success('编辑成功');
        } else {
            return error('编辑失败');
        }
    }

    public function group_delete()
    {
        $id = request()->post('id',0,'intval');
        $row = Db::table('cb_crm_attachment_group')->where('wid',$this->wid)->where('id',$id)->value('id');
        if (!$row) {
            return error('附件分组不存在');
        }
        $hasChildren = Db::table('cb_crm_attachment_group')->where('wid',$this->wid)->where('parent_id',$id)->value('id');
        if ($hasChildren) {
            return error('删除失败，请先删除下级分组');
        }

        $result = Db::table('cb_crm_attachment_group')->where('wid',$this->wid)->where('id',$id)->delete();
        if ($result) {
            return success('删除成功');
        } else {
            return error('删除失败');
        }
    }
}
