<?php


namespace app\cbcrm\controller\crm\trade;


use app\cbcrm\controller\BaseUser;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\trade\TrademarkLocalModel;
use app\cbcrm\model\trade\TrademarkOrderModel;
use app\cbcrm\services\TrademarkService;
use think\Request;

class TrademarkOrder extends BaseUser
{

    protected $model = null;

    protected $api_key = null;

    public function __construct(Request $request)
    {
        parent::__construct($request);

        $this->api_key = (new SettingModel())->getConfigValue('trade', 'trade_key', $this->wid);

        $this->model = new TrademarkOrderModel();
    }

    public function index()
    {
        $name = \request()->get('name');

        $status = \request()->get('status',-2);
        $pay_status = \request()->get('pay_status',-1);
        $params[] = ['trademark_order_model.wid','=',$this->wid];
        if ($name) {
            $params[] = ['trademark_order_model.name','LIKE',"%{$name}%"];
        }
        if ($status!=-2) {
            $params[] = ['trademark_order_model.status','=',$status];
        }
        if ($pay_status!=-1) {
            $params[] = ['trademark_order_model.pay_status','=',$pay_status];
        }


        $list = $this->model
            ->withJoin(['user'], 'LEFT')
            ->where($params)
            ->order('trademark_order_model.id desc')
            ->paginate(15);

        foreach ($list as &$item) {
            if ($item->tm_trade_certificate) {
                $item->tm_trade_certificate = cdnurl($item->tm_trade_certificate);
            }
        }

        return success('返回成功',$list);
    }

    public function edit()
    {
        $id = \request()->post('id',0,'intval');
        $params = \request()->post('form');
        $row = $this->model->where('id',$id)->where('wid',$this->wid)->find();
        if (!$row) {
            return error('未找到该内容');
        }
        if ($params['status']==1 && (!isset($params['tm_trade_certificate']) || !$params['tm_trade_certificate'])) {
            return error('交易成功请上传商标转让证明');
        }
        if ($params['status']==-1 && !$params['audit_memo']) {
            return error('交易失败请填写审核说明');
        }
        $row->save($params);
        $trademarkServices = app()->make(TrademarkService::class);
        if ($params['status']==1) {
            (new TrademarkLocalModel())->where('id',$row->local_id)->update([
                'trade_status'=>1,
                'admin_status'=>0,
            ]);
            $trademarkServices->updateCloudOrder($this->api_key,[
                'cloud_id'=>$row['cloud_id'],
                'unique_id' => $row['unique_id'],
                'pay_status' =>$row->pay_status,
                'status' => $params['status'],
                'tm_trade_certificate'=>$params['tm_trade_certificate']?cdnurl($params['tm_trade_certificate']):cdnurl($row->tm_trade_certificate)
            ]);
        }elseif($params['status']==-1){
            (new TrademarkLocalModel())->where('id',$row->local_id)->update([
                'trade_status'=>0,
                'admin_status'=>1,
            ]);
            $trademarkServices->updateCloudOrder($this->api_key,[
                'cloud_id'=>$row['cloud_id'],
                'unique_id' => $row['unique_id'],
                'pay_status' =>$row->pay_status,
                'status' => $params['status'],
            ]);
        }
        return success('编辑成功');
    }

    public function del()
    {
        $id = \request()->delete('id',0,'intval');
        $row = $this->model->where('id',$id)->where('wid',$this->wid)->find();
        if (!$row) {
            return error('未找到该内容');
        }
        $row->delete();
        return success('删除成功');
    }
}
