<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\copyright\OrderModel;
use app\cbcrm\model\copyright\ProductModel;
use app\cbcrm\model\goods\GoodsModel;
use app\cbcrm\model\goods\OrderDetailModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\model\trade\TrademarkOrderCostModel;
use app\cbcrm\services\PayService;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\Exception;
use think\facade\Db;
use think\facade\View;

class Copyright extends BaseSite
{

    public function __construct(\think\App $app)
    {
        parent::__construct($app);
        View::assign('title','版权登记');
    }

    public function index()
    {
        $list = (new ProductModel())->where('wid',$this->wid)->where('status',1)->select();

        foreach ($list as $key=>$item) {
            $list[$key]['tags'] = explode(',',$item['tag']);
            $list[$key]['spec'] = Db::table('cb_crm_copyright_sepc')
                ->where('delete_time',null)
                ->where('copyright_id',$item->id)
                ->field('name,price')
                ->order('price','asc')
                ->find();
        }

        View::assign('list',$list);

        return View::fetch('cbcrm@site/copyright/index');
    }

    public function detail()
    {
        $id = $this->request->get('id');

        $row = (new ProductModel())->where('id',$id)->where('status',1)->where('wid',$this->wid)->find();

        if (!$row) {
            abort(404,'该产品不存在或已下架');
        }

        $row->specList = Db::table('cb_crm_copyright_sepc')
            ->where('delete_time',null)
            ->where('copyright_id',$row->id)
            ->field('id,name,price')
            ->order('price','asc')
            ->select();

        $list = (new ProductModel())->where('wid',$this->wid)->where('status',1)->select();

        View::assign('list',$list);

        View::assign('row',$row);
        return View::fetch('cbcrm@site/copyright/detail');
    }

    public function order()
    {
        $id = $this->request->post('id');

        $spec_id = $this->request->post('spec_id',0,'intval');

        $row = (new ProductModel())->where('id',$id)->where('status',1)->where('wid',$this->wid)->find();

        if (!$row) {
            abort(404,'该产品不存在或已下架');
        }

        $row->spec = Db::table('cb_crm_copyright_sepc')
            ->where('delete_time',null)
            ->where('copyright_id',$row->id)
            ->where('id',$spec_id)
            ->field('id,name,price')
            ->find();

        View::assign('row',$row);

        View::assign('userInfo',$this->auth());

        return View::fetch('cbcrm@site/copyright/order');
    }

    public function pay()
    {
        $id = $this->request->post('id',0,'intval');
        $spec_id = $this->request->post('spec_id',0,'intval');
        $pay_type = $this->request->post('pay_type','wechat','strip_tags');
        $row = (new ProductModel())->where('id',$id)->where('status',1)->where('wid',$this->wid)->find();
        if (!$row) {
            abort(404,'该产品不存在或已下架');
        }
        $row->spec = Db::table('cb_crm_copyright_sepc')
            ->where('delete_time',null)
            ->where('copyright_id',$row->id)
            ->where('id',$spec_id)
            ->field('id,name,price')
            ->find();

        $result = false;
        $payData = false;
        Db::startTrans();
        try {
            $params = [];
            $params['wid'] = $this->wid;
            $params['user_id'] = $this->auth()->id;
            $params['order_no'] = date('YmdHis') . rand(10000, 99999);
            $params['copyright_id'] = $row->id;
            $params['spec_id'] = $spec_id;
            $params['name'] = $row->getAttr('name');
            $params['spec_attr'] = $row->spec['name'];
            $params['amount'] = $row->spec['price'];
            $params['pay_status'] = 0;
            $params['create_time'] = date('Y-m-d H:i:s');
            $params['update_time'] = date('Y-m-d H:i:s');

            $order_id = (new OrderModel())->insertGetId($params);

            $payService = app()->make(PayService::class);

            if ($pay_type == 'wechat') {
                $payData = $payService->payWechat($params['amount'],  $params['order_no'], $params['name'],$this->wid,'/copyright_order/wid/' . $this->wid . '/platform/PC');
                if (!$payData) {
                    throw new Exception('创建订单异常');
                }
                $payData = ['code_url'=>$payData['code_url']];
                $payData['amount'] = $params['amount'];
                $payData['order_no'] = $params['order_no'];
                $result = true;
            } elseif ($pay_type == 'balance') {
                $result = $payService->payBalance($params['amount'], $params['order_no'],$this->wid,$this->auth(),OrderModel::class,'版权登记');
                if ($result!==true) {
                    throw new Exception($result);
                }
                $order = (new OrderModel())->where('order_no', $params['order_no'])->find();

                // 修改订单支付状态
                $order->pay_status = 1;
                $order->pay_time = date('Y-m-d H:i:s');

                $order->save();
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result!==false) {
            $this->success('支付成功',$payData);
        }else{
            $this->error('支付失败');
        }
    }
}
