<?php
namespace app\cbcrm\controller\site;

use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\config\DictionaryModel;
use app\cbcrm\model\logo\LogoViModel;
use app\cbcrm\model\logo\PackageModel;
use app\cbcrm\model\logo\OrderDetailModel;
use app\cbcrm\model\logo\OrderModel;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\services\PayService;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use think\Exception;
use think\facade\Db;
use think\facade\View;

class Logo extends BaseSite
{
    public function __construct(\think\App $app)
    {
        parent::__construct($app);
        View::assign('title','LOGO设计');
    }
    public function index()
    {
        $list = (new PackageModel())->where('wid', $this->wid)->where('status', 1)->select();

        foreach ($list as $key => $item) {

            $list[$key]['feature'] = explode(',', $item['feature']);

            $list[$key]['standard'] = explode(',', $item['standard']);
        }

        // 行业
        $dictionary = (new DictionaryModel())->getTypeValues(87, $this->wid);

        //  配套VI
        $viList = (new LogoViModel())->where('wid', $this->wid)
            ->where('status', 1)
            ->order('sort desc, id asc')
            ->select();

        $viData = [
            'basic' => [],
            'company' => [],
            'store' => [],
        ];
        // 类型:0=基本VI,1=企业VI,2=店铺VI
        foreach ($viList as $vi) {
            switch ($vi['type']) {
                case 0:
                    $viData['basic'][] = $vi;
                    break;
                case 1:
                    $viData['company'][] = $vi;
                    break;
                case 2:
                    $viData['store'][] = $vi;
                    break;
            }
        }
        // 附加服务
        $otherService = Db::table('cb_crm_logo_other_service')->where('wid', $this->wid)->order('sort desc,id asc')->select();
        View::assign('list', $list);
        View::assign('dictionary', $dictionary);
        View::assign('otherService', $otherService);
        View::assign('viData', $viData);

        return View::fetch('cbcrm@site/logo/index');
    }

    public function order()
    {
        $package_id = $this->request->post('package_id',0,'intval')?:$this->error('参数错误');
        $logo_name = $this->request->post('logo_name');
        $category_id = $this->request->post('category_id');
        $introduce = $this->request->post('introduce');
        $idea = $this->request->post('idea');
        $other_service = $this->request->post('other_service/a');
        $case_files = $this->request->post('case_files');
        $idea_files = $this->request->post('idea_files');
        $vi = $this->request->post('vi/a');

        $row = (new PackageModel())
            ->where('id', $package_id)
            ->where('status', 1)
            ->where('wid', $this->wid)
            ->find();
        if (!$row) {
            $this->error('未找到产品');
        }
        $orderMoney = $row->price;
        // 加配套VI价格
        $vi_price = 0;
        if ($vi) {
            $orderMoney += $row->vi_price;
            $vi_price += $row->vi_price;
            $viCount = count($vi);
            if ($viCount > $row->vi_num) {
                $orderMoney += $row->unit_vi_price * ($viCount - $row->vi_num);
                $vi_price += $row->unit_vi_price * ($viCount - $row->vi_num);
            }
        }
        // 计算其他服务
        $otherServiceData = [];
        $other_service_price = 0;
        if (!empty($other_service)) {
            $otherServiceData = Db::table('cb_crm_logo_other_service')->where('wid',$this->wid)->where('id','IN',$other_service)->field('title as name,price')->select()->toArray();
            foreach ($otherServiceData as $item) {
                $orderMoney += $item['price'];
                $other_service_price += $item['price'];
            }
        }

        $detailData = [
            'logo_name'=>$logo_name,
            'category_id'=>$category_id,
            'case_files'=>$case_files,
            'introduce'=>$introduce,
            'idea'=>$idea,
            'idea_files'=>$idea_files
        ];

        $formData = $this->request->post();

        View::assign(compact('row','vi','otherServiceData','orderMoney','detailData','vi_price','other_service_price','formData'));

        return View::fetch('cbcrm@site/logo/order');
    }

    public function pay()
    {
        $package_id = $this->request->post('package_id',0,'intval')?:$this->error('参数错误');
        $logo_name = $this->request->post('logo_name');
        $category_id = $this->request->post('category_id');
        $introduce = $this->request->post('introduce');
        $idea = $this->request->post('idea');
        $other_service = $this->request->post('other_service/a');
        $case_files = $this->request->post('case_files');
        $idea_files = $this->request->post('idea_files');
        $vi = $this->request->post('vi/a');
        $pay_type = $this->request->post('pay_type');

        $row = (new PackageModel())
            ->where('id', $package_id)
            ->where('status', 1)
            ->where('wid', $this->wid)
            ->find();
        if (!$row) {
            $this->error('未找到产品');
        }
        $orderMoney = $row->price;
        // 加配套VI价格
        $vi_price = 0;
        if ($vi) {
            $orderMoney += $row->vi_price;
            $vi_price += $row->vi_price;
            $viCount = count($vi);
            if ($viCount > $row->vi_num) {
                $orderMoney += $row->unit_vi_price * ($viCount - $row->vi_num);
                $vi_price += $row->unit_vi_price * ($viCount - $row->vi_num);
            }
        }
        // 计算其他服务
        $otherServiceData = [];
        $other_service_price = 0;
        if (!empty($other_service)) {
            $otherServiceData = Db::table('cb_crm_logo_other_service')->where('wid',$this->wid)->where('id','IN',$other_service)->field('title as name,price')->select()->toArray();
            foreach ($otherServiceData as $item) {
                $orderMoney += $item['price'];
                $other_service_price += $item['price'];
            }
        }
        $orderNo = date('YmdHis') . rand(10000, 99999);
        $user_id = $this->auth()->id;
        $result = false;
        $payData = false;
        Db::startTrans();
        try {
            // 创建订单
            $orderData = [
                'wid'=>$this->wid,
                'user_id'=>$user_id,
                'order_no'=>$orderNo,
                'price'=>$orderMoney,
                'pay_status'=>0,
                'status'=>0,
                'package_id'=>$row->id,
                'package_info'=>json_encode($row,JSON_UNESCAPED_UNICODE),
                'create_time'=>date('Y-m-d H:i:s'),
                'update_time'=>date('Y-m-d H:i:s')
            ];
            $order_id = (new OrderModel())->createOrder($orderData);
            // 订单详情
            $orderDetailData = [
                'wid'=>$this->wid,
                'order_id'=>$order_id,
                'logo_name'=>$logo_name,
                'category_id'=>$category_id,
                'case_files'=>$case_files,
                'introduce'=>$introduce,
                'idea'=>$idea,
                'idea_files'=>$idea_files,
                'vi'=>$vi?trim(implode(',',$vi),','):'',
                'other_service'=>json_encode($otherServiceData,JSON_UNESCAPED_UNICODE)
            ];
            (new OrderDetailModel())->createOrderDetail($orderDetailData);

            $payService = app()->make(PayService::class);

            if ($pay_type == 'wechat') {
                $payData = $payService->payWechat($orderData['price'],  $orderData['order_no'], $logo_name,$this->wid,'/logo_order/wid/' . $this->wid . '/platform/PC');
                if (!$payData) {
                    throw new Exception('创建订单异常');
                }
                $payData = ['code_url'=>$payData['code_url']];
                $payData['amount'] = $orderData['price'];
                $payData['order_no'] = $orderData['order_no'];
                $result = true;
            } elseif ($pay_type == 'balance') {
                $result = $payService->payBalance($orderData['price'], $orderData['order_no'],$this->wid,$this->auth(), OrderModel::class,'LOGO设计');
                if ($result!==true) {
                    throw new Exception($result);
                }
                $order = (new OrderModel())->where('order_no', $orderData['order_no'])->find();
                // 修改订单支付状态
                $order->pay_status = 1;
                $order->pay_time = date('Y-m-d H:i:s');
                $order->save();
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        $this->success('支付成功',$payData);
    }
}
