<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\trademark\FileFetchModel;
use app\cbcrm\model\trademark\FileModel;
use think\App;
use think\Exception;
use think\facade\Db;
use think\facade\View;

class TmFile extends BaseSite
{
    protected $model = null;
    public function __construct(App $app)
    {
        $this->model = new FileModel();
        View::config(['layout_on'=>true,'layout_name' => 'site/common/user_layout']);
        parent::__construct($app);
    }

    public function index()
    {
        $user_id = $this->auth()->id;
        $type = $this->request->param('type','all');
        if ($type!='record') {
            $where = [
                'user_id'=>$user_id
            ];
            if ($type=='wait') {
                $where['type'] = 1;
                $where['status']= 0;
            }
            $list = (new FileModel())
                ->with(['trademark','user'])
                ->where($where)
                ->order('id', 'desc')
                ->paginate(10, false, ['query' => get_query()]);

            foreach ($list as $row) {
                $row->visible(['id','user_id','tm_id','order_id','type','file_type','status','createtime']);
                $row->visible(['trademark']);
                $row->getRelation('trademark')->visible(['name','category']);
                $row->visible(['user']);
                $row->getRelation('user')->visible(['nickname']);
            }
        }elseif($type=='record'){
            $fetchModel = new FileFetchModel();
            $list = $fetchModel
                ->with(['user'])
                ->where(['user_id'=>$user_id])
                ->order('id','desc')
                ->paginate(10, false, ['query' => get_query()]);
            foreach ($list as $row) {
                $row->visible(['user']);
                $row->getRelation('user')->visible(['nickname']);
            }
        }

        View::assign('type', $type);
        View::assign('list', $list);
        View::assign('page', $list->render());
        return View::fetch('cbcrm@site/user/tm/file/index');
    }

    public function file_fetches()
    {
        $user_id = $this->auth()->id;
        if ($this->request->isPost()) {
            $params = $this->request->post('row/a');

            if (!$params['name']) {
                $this->error('请填写收件人姓名');
            }
            if (!$params['mobile']) {
                $this->error('请填写收件人手机号码');
            }
            if (!$params['city']) {
                $this->error('请填写收件人所在地区');
            }
            if (!$params['address']) {
                $this->error('请填写收件人所在地详细信息');
            }
            $ids = implode(',',$params['ids']);
            unset($params['ids']);
            if (!$ids) {
                $this->error('请选择要索取的发文');
            }
            $fetchModel = new FileFetchModel();

            $result = false;
            Db::startTrans();
            try {
                $fileList = $this->model->where('id','IN',$ids)->where('type',1)->where('status',0)->where('user_id',$user_id)->select();
                $params['user_id'] = $user_id;
                $params['wid'] = $this->wid;
                $params['file_ids'] = [];
                foreach ($fileList as $item) {
                    array_push($params['file_ids'],$item->id);
                    $this->model->where('id',$item->id)->update(['status'=>1]);
                }
                $params['file_ids'] = implode(',',$params['file_ids']);
                $params['status'] = 0;
                if ($fileList) {
                    $result = $fetchModel->save($params);
                }
                Db::commit();
            } catch (Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if ($result) {
                $this->success('提交申请成功');
            }else{
                $this->error('提交申请失败');
            }
        }
        $where = [
            'user_id'=>$user_id,
            'type'=>1,
            'status'=>0
        ];
        $list = $this->model
            ->with(['trademark','user'])
            ->where($where)
            ->order('id', 'desc')
            ->paginate(10, false, ['query' => get_query()]);

        foreach ($list as $row) {
            $row->visible(['id','user_id','tm_id','order_id','type','file_type','status','createtime']);
            $row->visible(['trademark']);
            $row->getRelation('trademark')->visible(['name','category']);
            $row->visible(['user']);
            $row->getRelation('user')->visible(['nickname']);
        }

        View::assign('list', $list);
        View::assign('page', $list->render());
        View::assign('title','商标发文索取');
        return View::fetch('cbcrm@site/user/tm/file/file_fetches');
    }

    public function show_fetch()
    {
        $user_id = $this->auth()->id;
        $id = $this->request->param('id');
        $fetchModel = new FileFetchModel();
        $row = $fetchModel->where(['user_id'=>$user_id,'id'=>$id])->find();
        if (!$row) {
            $this->error('记录未找到');
        }

        $list = $this->model
            ->where('id','IN',$row->file_ids)
            ->where('user_id',$user_id)
            ->select();

        View::assign('list',$list);

        View::assign('row',$row);
        View::assign('title','索取发文详情');
        return View::fetch('cbcrm@site/user/tm/file/show_fetch');
    }

    public function cancel_fetch()
    {
        if ($this->request->isAjax()) {
            $id = $this->request->param('id',0,'intval');
            $fetchModel = new FileFetchModel();
            $row = $fetchModel->where(['user_id'=>$this->auth()->id,'id'=>$id])->find();
            if (!$row) {
                $this->error('记录未找到');
            }

            if ($row->status==3) {
                $this->error('该发文已邮寄，无法进行操作');
            }

            $row->save(['status'=>4]);

            $res = $this->model->where('id','IN',$row->file_ids)->update(['status'=>0]);

            if ($res) {
                $this->success('操作成功');
            }else{
                $this->error('操作失败');
            }
        }
    }
}
