<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\trademark\OrderModel;
use think\App;
use think\facade\Db;
use think\facade\View;

class TmOrder extends BaseSite
{
    public function __construct(App $app)
    {
        View::config(['layout_on'=>true,'layout_name' => 'site/common/user_layout']);
        parent::__construct($app);
    }

    public function index()
    {
        $list = Db::table('cb_crm_tm_order')
            ->alias('o')
            ->join(['cb_crm_tm_applicants'=>'a'],'o.applyer_id = a.id')
            ->join(['cb_crm_tm_product'=>'p'],'o.product_id = p.id')
            ->where('o.user_id', $this->auth()->id)
            ->where('o.delete_time', null)
            ->field('o.*,a.name as applyer_name,p.name as product_name')
            ->order('o.id', 'desc')
            ->paginate(10);

        $tmClasses = file_get_contents($this->request->domain().$this->__CDN__.'/js/tm-classes-alias.js');
        $tmClasses = json_decode(trim(ltrim($tmClasses,'module.exports ='),';'),JSON_UNESCAPED_UNICODE);
        $temp = [];
        foreach ($tmClasses as $key=>$class) {
            $temp[$key] = $class;
        }
        $tmClasses = $temp;
        View::assign('list', $list);
        View::assign('page', $list->render());
        View::assign( 'tmClassesList',$tmClasses);
        return View::fetch('cbcrm@site/user/tm/order/index');
    }

    public function detail()
    {
        $id = $this->request->param('id');
        $row = (new OrderModel())->where(['id'=>$id,'user_id'=>$this->auth()->id])->find();
        if (!$row) {
            $this->error('订单不存在');
        }

        $row->goods_data = json_decode($row->goods_data,true);

        View::assign('row',$row);

        return View::fetch('cbcrm@site/user/tm/order/detail');
    }

    /*
     * 删除订单
     * */
    public function del()
    {
        if ($this->request->isAjax()) {
            $user_id = $this->auth()->id;
            $id = $this->request->param('id',0,'intval')?:$this->error('参数错误');
            $row = Db::table('cb_crm_tm_order')->where('id',$id)->where('user_id',$user_id)->find();
            if (!$row) {
                $this->error('未找到订单');
            }
            $res = Db::table('cb_crm_tm_order')->where('id',$id)->where('user_id',$user_id)->update(['delete_time'=>time()]);
            if (!$res) {
                $this->error('删除失败');
            }else{
                $this->success('删除成功');
            }
        }
    }
}
