<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\trade\TrademarkLocalModel;
use app\cbcrm\model\trade\TrademarkOrderModel;
use app\cbcrm\model\trade\TrademarkOrderCostModel;
use app\cbcrm\services\PayService;
use app\cbcrm\services\TrademarkService;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use think\App;
use think\Exception;
use think\facade\Db;
use think\facade\Log;
use think\facade\View;
use think\Request;

class TmTradeLib extends BaseSite
{
    protected $apiUrl = 'http://haoip.tunke.cn/api';

    protected $api_key = '';

    protected $trade_money_ratio = 0;

    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->api_key = (new SettingModel())->getConfigValue('trade', 'trade_key', $this->wid);

        $this->trade_money_ratio = (new SettingModel())->getConfigValue('trade', 'trade_money_ratio', $this->wid);
    }

    public function index()
    {
        if ($this->request->isAjax()) {
            $page = \request()->get('page', 1, 'intval');
            $search_type = \request()->get('search_type');
            $search = \request()->get('search', '', 'strip_tags,trim,htmlspecialchars');
            $cls = \request()->get('cls', -1);
            $type = \request()->get('type','');
            $price = \request()->get('price','');
            $trademark_length = \request()->get('trademark_length', '');
            $params = [
                'api_key' => $this->api_key,
                'page' => $page,
                'status' => 1,
                'trade_status' => 0,
                'admin_status' => 1,
                'limit' => 40
            ];
            if ($search) {
                $params['search_type'] = $search_type;
                $params['search'] = $search;
            }
            if ($cls > -1) {
                $params['cls'] = $cls;
            }
            if ($trademark_length) {
                $params['trademark_length'] = $trademark_length;
            }
            if ($type) {
                $params['type'] = $type;
            }
            if ($price) {
                // 0-9999
                $priceArr = explode('-',$price);
                if (count($priceArr) > 1) {
                    if ($priceArr[0] > $priceArr[1]) {
                        $this->error('最大金额必须大于最小金额');
                    }
                    if ($this->trade_money_ratio) {
                        if ($priceArr[0] > 0) {
                            $priceArr[0] =  round($priceArr[0] / (1+($this->trade_money_ratio / 100)));
                        }
                        $priceArr[1] = round($priceArr[1] / (1+($this->trade_money_ratio / 100)));
                        $price = implode('-',$priceArr);
                    }
                }
                $params['price'] = $price;
            }
            $data = [
                'current_page' => 1,
                'data' => [],
                'last_page' => 1,
                'per_page' => 1,
                'total' => 0,
            ];
            $client = new Client();
            try {
                $response = $client->request('GET', $this->apiUrl . '/trade/trademark/getCloudList', [
                    'query' => $params
                ]);
                $result = json_decode($response->getBody(), true);
                if ($result['code'] != 1) {
                    throw new Exception($result['msg']);
                }
                $list = $result['data']['list'];
                $model = app()->make(TrademarkLocalModel::class);
                foreach ($list as &$item) {
                    $item['range'] = subtext($item['range'], 30);
                    $item['name'] = subtext($item['name'], 19);
                    if (session('memberToken')) {
                        // 自有商标
                        if ($item['key'] && $item['key'] == $this->api_key) {
                            $localRow = $model->where('unique_id',$item['unique_id'])->find();
                            $item['price'] = number_format($localRow['price']);
                            $item['sale_price'] = number_format($localRow['sale_price']);
                        }else {
                            // 云商标
                            $item['price'] = $item['price'] / 100;
                            if ($this->trade_money_ratio) {
                                $item['sale_price'] = number_format($item['price'] + round($item['price'] * ($this->trade_money_ratio / 100)));
                            } else {
                                $item['sale_price'] = number_format($item['price']);
                            }
                            $item['price'] = number_format($item['price']);

                        }
                    } else {
                        $item['price'] = '请登录后查看';
                        $item['sale_price'] = '请登录后查看';
                    }
                }
                $data = [
                    'current_page' => $page,
                    'data' => $list,
                    'last_page' => ceil($result['data']['total'] / 40),
                    'per_page' => 40,
                    'total' => $result['data']['total'],
                ];
            } catch (GuzzleException $e) {
                $this->error($e->getMessage());
            } catch (Exception $e) {
                $this->error($e->getMessage());
            }
            $this->success('返回成功',$data);
        }

        View::assign('trademarkData');
        return View::fetch('cbcrm@site/trade/trademark/index');
    }

    public function detail()
    {
        $id = $this->request->get('id',0,'intval')?:$this->error('参数错误');

        $services = app()->make(TrademarkService::class);

        $row =  $services->getCloudLibDetail($id,$this->api_key);

        if (!$row || $row['status']!=1 || $row['admin_status']!=1 || $row['trade_status']!=0) {
            $this->error('商标未找到');
        }

        $model = app()->make(TrademarkLocalModel::class);

        if ($row['key'] && $row['key'] == $this->api_key) {
            // 自有商标
            $localRow = $model->where('unique_id',$row['unique_id'])->where('wid',$this->wid)->find();
            if ($localRow) {
                $row['price'] = number_format($localRow['price']);
                $row['sale_price'] = number_format($localRow['sale_price']);
            }
        }else{
            $row['price'] = $row['price'] / 100;
            if ($this->trade_money_ratio) {
                $row['sale_price'] = number_format($row['price'] + round($row['price'] * ($this->trade_money_ratio / 100)));
            } else {
                $row['sale_price'] = number_format($row['price']);
            }
            $row['price'] = number_format($row['price']);
        }

        View::assign('row',$row);

        return View::fetch('cbcrm@site/trade/trademark/detail');
    }

    public function order()
    {
        $id = $this->request->get('id',0,'intval')?:$this->error('参数错误');

        $services = app()->make(TrademarkService::class);

        $row =  $services->getCloudLibDetail($id,$this->api_key);

        if (!$row || $row['status']!=1 || $row['admin_status']!=1 || $row['trade_status']!=0) {
            $this->error('商标未找到');
        }

        $model = app()->make(TrademarkLocalModel::class);

        if ($row['key'] && $row['key'] == $this->api_key) {
            // 自有商标
            $localRow = $model->where('unique_id',$row['unique_id'])->where('wid',$this->wid)->find();
            if ($localRow) {
                $row['price'] = $localRow['price'];
                $row['sale_price'] = $localRow['sale_price'];
            }
        }else{
            $row['price'] = $row['price'] / 100;
            if ($this->trade_money_ratio) {
                $row['sale_price'] = $row['price'] + round($row['price'] * ($this->trade_money_ratio / 100));
            } else {
                $row['sale_price'] = $row['price'];
            }
        }

        // 商标交易服务费
        $trade_service_price = (new SettingModel())->getConfigValue('trade', 'trade_service_price', $this->wid);

        $row['trade_service_price'] = $trade_service_price;

        // 定金 = 出售价格 * 15%；
        $row['deposit_price'] = $row['sale_price'] * 0.15;

        View::assign('row',$row);

        return $this->view('trade/trademark/order');
    }

    /*
     * 定金支付
     * */
    public function pay()
    {
        $id = $this->request->post('id',0,'intval')?:$this->error('参数错误');
        $pay_type = $this->request->post('pay_type','wechat');
        $receive_name = $this->request->post('receive_name')?:$this->error('请填写联系人');
        $receive_phone = $this->request->post('receive_phone')?:$this->error('请填写联系电话');
        $memo = $this->request->post('memo');

        $services = app()->make(TrademarkService::class);

        $row =  $services->getCloudLibDetail($id,$this->api_key);

        if (!$row || $row['status']!=1 || $row['admin_status']!=1 || $row['trade_status']!=0) {
            $this->error('商标未找到');
        }

        $model = app()->make(TrademarkLocalModel::class);
        $localRow = null;
        if ($row['key'] && $row['key'] == $this->api_key) {
            // 自有商标
            $localRow = $model->where('unique_id',$row['unique_id'])->where('wid',$this->wid)->find();
            if ($localRow) {
                $row['price'] = $localRow['price'];
                $row['sale_price'] = $localRow['sale_price'];
            }
        }else{
            $row['price'] = $row['price'] / 100;
            if ($this->trade_money_ratio) {
                $row['sale_price'] = $row['price'] + round($row['price'] * ($this->trade_money_ratio / 100));
            } else {
                $row['sale_price'] = $row['price'];
            }
        }

        // 商标交易服务费
        $trade_service_price = (new SettingModel())->getConfigValue('trade', 'trade_service_price', $this->wid);

        $row['trade_service_price'] = $trade_service_price;

        // 定金 = 出售价格 * 15%；
        $row['deposit_price'] = $row['sale_price'] * 0.15;

        $result = false;
        $payData = false;
        Db::startTrans();
        try {
            $params = [
                'wid' => $this->wid,
                'user_id' => $this->auth()->id,
                'cloud_id' => $row['id'],
                'tm_id' => $row['tm_id'],
                'cls' => $row['cls'],
                'cls_name' => $row['cls_name'],
                'name' => $row['name'],
                'tm_trade_image' => $row['tm_trade_image'],
                'tm_price' => $row['sale_price'],
                'deposit_price' => $row['deposit_price'],
                'service_price' => $row['trade_service_price'],
                'receive_name' => $receive_name,
                'receive_phone' => $receive_phone,
                'memo' => $memo,
                'type' => $localRow?1:0,
                'local_id' => $localRow?$localRow->id:null,
                'pay_status' => 0,
                'pay_type' => $pay_type,
                'status' => 0,
                'create_time' => date('Y-m-d H:i:s'),
                'update_time' => date('Y-m-d H:i:s'),
            ];

            $orderModel = app()->make(TrademarkOrderModel::class);

            $order_id = $orderModel->insertGetId($params);

            $orderCostModel = app()->make(TrademarkOrderCostModel::class);

            // 定金费用
            $depositParams = [
                'order_id'=>$order_id,
                'order_no' => date('YmdHis') . rand(10000, 99999),
                'price'=>$row['deposit_price'],
                'type'=>1,
                'pay_status'=>0,
                'pay_type'=>$pay_type
            ];
            $orderCostModel->insert($depositParams);

            // 尾款+服务费
            $balanceParams = [
                'order_id'=>$order_id,
                'order_no' => date('YmdHis') . rand(10000, 99999),
                'price'=>$row['sale_price'] - $row['deposit_price'] + $row['trade_service_price'],
                'type'=>2,
                'pay_status'=>0,
                'pay_type'=>$pay_type
            ];

            $orderCostModel->insert($balanceParams);

            $payParams = [
                'amount'=>$depositParams['price'],
                'order_no'=>$depositParams['order_no'],
                'name'=>$row['name'].' - 商标交易定金',
                'notify_url'=>'/tm_trade_order/wid/'.$this->wid.'/platform/PC'
            ];

            $payService = app()->make(PayService::class);

            if ($pay_type == 'wechat') {
                $payData = $payService->payWechat($payParams['amount'],  $payParams['order_no'], $payParams['name'],$this->wid,$payParams['notify_url']);
                if (!$payData) {
                    throw new Exception('创建订单异常');
                }
                $payData = ['code_url'=>$payData['code_url']];
                $payData['amount'] = $payParams['amount'];
                $payData['order_no'] = $payParams['order_no'];
                $result = true;
            } elseif ($pay_type == 'balance') {
                $result = $payService->payBalance($payParams['amount'], $payParams['order_no'],$this->wid,$this->auth(),TrademarkOrderCostModel::class,'商标交易');

                if ($result!==true) {
                    throw new Exception($result);
                }else{
                    (new TrademarkOrderModel())->where('id',$order_id)->update([
                        'pay_status'=>1,
                        'pay_time'=>date('Y-m-d H:i:s'),
                        'status'=>2
                    ]);
                    $orderCostModel->where('order_no',$payParams['order_no'])->update([
                        'pay_status'=>1,
                        'pay_type'=>$pay_type,
                        'pay_time'=>date('Y-m-d H:i:s'),
                    ]);
                    // 更新自有商标交易库
                    if ($localRow) {
                        $localRow->admin_status = 0;
                        $localRow->save();
                    }
                    // 创建云交易库订单
                    $trademarkServices = app()->make(TrademarkService::class);
                    $data = (new TrademarkOrderModel())->where('id',$order_id)->find();
                    $trademarkServices->createCloudOrder($order_id,$this->api_key,$data);
                }
            }

            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result!==false) {
            $this->success('支付成功',$payData);
        }else{
            $this->error('支付失败');
        }

    }

    /*
     * 支付尾款和服务
     * */
    public function pay_balance()
    {
        $user = $this->auth();
        $order_id = $this->request->post('order_id')?:$this->error('参数错误');
        $pay_type = $this->request->post('pay_type','wechat');

        $orderModel = app()->make(TrademarkOrderModel::class);

        $row = $orderModel->where('id',$order_id)->where('user_id',$user->id)->find();

        if (!$row || $row->pay_status!=1) {
            $this->error('待支付订单不存在');
        }

        $orderCostModel = app()->make(TrademarkOrderCostModel::class);

        $balance_cost = $orderCostModel->where('order_id',$order_id)->where('type',2)->find();

        if (!$balance_cost) {
            $this->error('待支付的尾款不存在');
        }
        $result = false;
        $payData = false;
        Db::startTrans();
        try {

            $payParams = [
                'amount'=>$balance_cost['price'],
                'order_no'=>$balance_cost['order_no'],
                'name'=>$row['name'].' - 商标尾款和服务费',
                'notify_url'=>'/tm_trade_order/wid/'.$row->wid.'/platform/PC'
            ];

            $payService = app()->make(PayService::class);

            if ($pay_type == 'wechat') {
                $payData = $payService->payWechat($payParams['amount'],  $payParams['order_no'], $payParams['name'],$row->wid,$payParams['notify_url']);
                if (!$payData) {
                    throw new Exception('创建订单异常');
                }
                $payData = ['code_url'=>$payData['code_url']];
                $payData['amount'] = $payParams['amount'];
                $payData['order_no'] = $payParams['order_no'];
                $result = true;
            } elseif ($pay_type == 'balance') {
                $result = $payService->payBalance($payParams['amount'], $payParams['order_no'],$row->wid,$user,TrademarkOrderCostModel::class,'商标交易');

                if ($result!==true) {
                    throw new Exception($result);
                }else{
                    (new TrademarkOrderModel())->where('id',$order_id)->update([
                        'pay_status'=>2,
                        'pay_time'=>date('Y-m-d H:i:s'),
                        'status'=>4
                    ]);
                    $orderCostModel->where('order_no',$payParams['order_no'])->update([
                        'pay_status'=>1,
                        'pay_time'=>date('Y-m-d H:i:s'),
                    ]);
                    // 更新云交易库订单
                    $trademarkServices = app()->make(TrademarkService::class);
                    $trademarkServices->updateCloudOrder($this->api_key,[
                            'cloud_id'=>$row['cloud_id'],
                            'unique_id'=>$row['unique_id'],
                            'pay_status'=>2,
                            'status'=>4
                    ]);
                }
            }

            Db::commit();
        }catch (Exception $e){
            Db::rollback();
            $this->error($e->getMessage());
        }

        if ($result!==false) {
            $this->success('支付成功',$payData);
        }else{
            $this->error('支付失败');
        }


    }
}
