<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\logo\PackageModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\model\trademark\ApplyerModel;
use app\cbcrm\model\trademark\OrderModel;
use app\cbcrm\model\trademark\ProductModel;
use app\cbcrm\services\PayService;
use app\cbcrm\services\SmsServices;
use EasyWeChat\Factory;
use GuzzleHttp\Client;
use think\Exception;
use think\facade\Db;
use think\facade\View;

class Trademark extends BaseSite
{
    protected $tunkeUrl = 'https://www.tunke.cn';

    public function __construct(\think\App $app)
    {
        parent::__construct($app);
        View::assign('title','商标注册');
    }

    public function index()
    {
        $tmList = (new ProductModel())
            ->where('wid',$this->wid)
            ->where('status',1)
            ->where('mode',0)
            ->order('sort desc,id desc')
            ->field('id,name,image,price,mode,describe,advantage')
            ->limit(0,2)
            ->select();
        foreach ($tmList as $key=>$item) {
            $tmList[$key]['describe'] = subtext($item['describe'],40);
            $tmList[$key]['advantage'] = explode(',',$item['advantage']);
        }

        $logoProduct = (new PackageModel())->where('wid',$this->wid)->where('status',1)
            ->field('id,title,price,feature,standard')
            ->order('price asc,sort desc')
            ->find();

        if ($logoProduct) {
            $logoProduct->feature = explode(',',$logoProduct->feature);
        }

        View::assign(compact('tmList','logoProduct'));

        return View::fetch('cbcrm@site/trademark/index');
    }

    public function order(){
        $id = $this->request->get('id',0,'intval')?:$this->error('参数错误');

        $row = (new ProductModel())->where('id',$id)->where('wid',$this->wid)->where('status',1)->find();

        if (!$row) {
            $this->error('产品未找到');
        }
        if($this->request->isPost()){
            $applyer_id = $this->request->post('applyer_id',0,'intval')?:$this->error('请选择申请主体');
            $name = $this->request->post('name');
            $description= $this->request->post('description');
            $registration_delegation_file = $this->request->post('registration_delegation_file','');
            $registration_image_file = $this->request->post('registration_image_file','')?:$this->error('请上传商标图样');
            $this->request->filter(false);
            $productList = $this->request->post('product','',false)?:$this->error('请选择商标类别');
            $order_no = date('YmdHis').rand(10000,99999);
            Db::startTrans();
            try {
                $productData = json_decode($productList,true);
                foreach ($productData as $key=>$product) {
                    $data = [
                        'user_id'=>$this->auth()->id,
                        'wid'=>$this->wid,
                        'order_no'=>$order_no,
                        'product_id'=>$id,
                        'name'=>$name,
                        'description'=>$description,
                        'category'=>$product['category']['id'],
                        'applyer_id'=>$applyer_id,
                        'registration_delegation_file'=>$registration_delegation_file,
                        'registration_image_file'=>$registration_image_file,
                        'pay_status'=>0,
                        'status'=>'待审核',
                        'create_time'=>date('Y-m-d H:i:s'),
                        'update_time'=>date('Y-m-d H:i:s'),
                    ];
                    $goods_data = [];
                    $goodsData = $product['goodsData'];
                    foreach ($goodsData as $goods) {
                        $items = $goods['goodsData'];
                        foreach ($items as $item) {
                            $item['category_id'] = $product['category']['id'];
                            unset($item['remark']);
                            array_push($goods_data,$item);
                        }
                    }
                    $data['goods_data'] = json_encode($goods_data,JSON_UNESCAPED_UNICODE);
                    if (count($goods_data)<=10) {
                        $data['amount'] = $row->price;
                    }else{
                        $data['amount'] = $row->price + ((count($goods_data) - 10) * $row->unit_price);
                    }
                    Db::table('cb_crm_tm_order')->insert($data);

                    Db::table('cb_crm_tm_applicants')->where(['id'=>$applyer_id,'user_id'=>$this->auth()->id])->inc('tm_count',1)->update();
                }
                Db::commit();
            } catch (Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            $this->success('提交生成，请确认订单并支付',['order_no'=>$order_no]);
        }

        // 商标申请主体
        $tmApplyer = Db::table('cb_crm_tm_applicants')->where('status','<>',2)->where('user_id',$this->auth()->id)->order('id','desc')->select();

        $tmClasses = file_get_contents($this->request->domain().$this->__CDN__.'/js/tm-classes-alias.js');
        $tmClasses = json_decode(trim(ltrim($tmClasses,'module.exports ='),';'),JSON_UNESCAPED_UNICODE);
        $temp = [];
        foreach ($tmClasses as $key=>$class) {
            $temp[] = [
                'id'=>$key,
                'name'=>$class
            ];
        }
        $tmClasses = $temp;
        View::assign('tmClasses',$tmClasses);

        View::assign('tmApplyer',$tmApplyer);
        View::assign('trademarkInfo',$row);
        View::assign('user',$this->auth());

        return View::fetch('cbcrm@site/trademark/order');
    }

    public function pay()
    {
        $order_no = $this->request->request('order_no');
        $list = Db::table('cb_crm_tm_order')->where('order_no',$order_no)->where('user_id',$this->auth()->id)->select()->toArray();

        if (!count($list)>0) {
            $this->error('订单不存在');
        }
        $trademarkInfo = [];
        foreach ($list as $key=>$row) {
            if ($row['pay_status']!=0) {
                $this->error('该订单已支付');
            }
            $trademarkInfo['order_no'] = $row['order_no'];
            $trademarkInfo['applyer_id'] = $row['applyer_id'];
            $trademarkInfo['name'] = $row['name'];
            $trademarkInfo['description'] = $row['description'];
            $trademarkInfo['registration_delegation_file'] = $row['registration_delegation_file'];
            $trademarkInfo['registration_image_file'] = $row['registration_image_file'];
            $list[$key]['goods_data'] = json_decode($row['goods_data'],true);
            if (isset($trademarkInfo['amount'])) {
                $trademarkInfo['amount'] += $row['amount'];
            }else{
                $trademarkInfo['amount'] = $row['amount'];
            }
        }

        if ($this->request->isPost()) {
            $pay_type = $this->request->post('pay_type', 'wechat', 'strip_tags');
            $result = false;
            $payData = false;
            Db::startTrans();
            try {
                //订单标题
                $title = $trademarkInfo['name'] . ' - ' . '商标注册';
                $amount = $trademarkInfo['amount']; //
                $payService = app()->make(PayService::class);
                if ($pay_type == 'wechat') {
                    $payData = $payService->payWechat($amount, $order_no, $title,$this->wid,'/trademerk_order/wid/' . $this->wid . '/platform/PC');
                    if (!$payData) {
                        throw new Exception('创建订单异常');
                    }
                    $payData = ['code_url' => $payData['code_url']];
                    $payData['amount'] = $amount;
                    $payData['order_no'] = $order_no;
                    $result = true;
                } elseif ($pay_type == 'balance') {
                    $result = $payService->payBalance($amount,$order_no,$this->wid,$this->auth(), OrderModel::class,'商标注册');
                    if ($result!==true) {
                        throw new Exception($result);
                    }
                    $order = (new OrderModel())->where('order_no', $order_no)->find();
                    $data = [
                        'pay_status'=>1,
                        'pay_time'=>date('Y-m-d H:i:s')
                    ];
                    // 修改订单支付状态
                    $res = (new OrderModel())->where('order_no', $order_no)->update($data);
                }
                Db::commit();
            } catch (Exception $e) {
                Db::rollback();
                $this->error($e->getMessage());
            }
            if ($result !== false) {
                $this->success('支付成功', $payData);
            } else {
                $this->error('支付失败');
            }
        }


        $applyerInfo = (new ApplyerModel())->where('id',$trademarkInfo['applyer_id'])->where('user_id',$this->auth()->id)->find();

        View::assign('trademarkInfo',$trademarkInfo);

        View::assign('applyerInfo',$applyerInfo);

        View::assign('list',$list);

        return View::fetch('cbcrm@site/trademark/pay');

    }

    public function search()
    {
        $type = $this->request->get('search_type',1,'intval');
        $keyword = $this->request->get('q','','strip_tags');
        $class_id = $this->request->get('class_id','','strip_tags');
        $page = $this->request->get('page',1);
        $page_size = $this->request->get('page_size',20);
        $stage = $this->request->get('stage','');

        if (!$keyword) {
            $this->error('参数错误');
        }

        // 处理查询类别
        $class_id = trim($class_id,';');
        $classArr = [];
        if ($class_id) {
            $classArr = explode(';',$class_id);
            foreach ($classArr as $key=>$value) {
                $classArr[$key] = (int)$value;
            }
            $class_id = implode(';',$classArr);
        }
        $client = new Client();

        $data = [
            'keyword'=>$keyword,
            'class_id'=>$class_id,
            'type'=>$type,
            'stage'=>$stage,
            'page'=>$page,
            'page_size'=>$page_size,
            'domain'=>request()->domain()
        ];
        $response = $client->get($this->tunkeUrl . '/v1/tm/crm_list_search', [
            'query' => $data
        ]);
        $result = $response->getBody();
        $jsonData = \GuzzleHttp\json_decode($result, true);

        $resultData = isset($jsonData['data']['data'])?$jsonData['data']['data']:[];

        $total = $resultData['total'];
        $trademarkList = $resultData['data'];

        $totalPage = ceil($total/$page_size);

        View::assign(compact('total','class_id','classArr','trademarkList','type','total','page','totalPage','keyword'));

        return View::fetch('cbcrm@site/trademark/search');
    }

    public function search_details()
    {
        $regNo = $this->request->get('regNo');
        $class_id = $this->request->get('class_id');

        if (!$regNo || !$class_id) {
            $this->error('参数错误');
        }

        $client = new Client();

        $data = [
            'tm_id'=>$regNo, // 申请号
            'class_id'=>$class_id, // 类别
            'domain'=>request()->domain()
        ];
        $response = $client->get($this->tunkeUrl . '/v1/tm/crm_detail_search', [
            'query' => $data
        ]);
        $result = $response->getBody();
        $jsonData = \GuzzleHttp\json_decode($result, true);



        $resultData = isset($jsonData['data']['data'])?$jsonData['data']['data']:[];


        $trademarkInfo = isset($resultData['data'])?$resultData['data']:'';

        $noticeData = (isset($resultData['noticeData']) && isset($resultData['noticeData']['data']))?$resultData['noticeData']['data']:[];

       View::assign('trademarkInfo',$trademarkInfo);
       View::assign('noticeData',$noticeData);
       return View::fetch('cbcrm@site/trademark/search_details');
    }

    public function company_detail()
    {
        $keyword = $this->request->get('keyword','','strip_tags');
        $page = $this->request->get('page',1,'intval');
        $page_size = $this->request->get('page_size',20);

        if (!$keyword) {
            $this->error('参数错误');
        }

        $client = new Client();

        $data = [
            'keyword'=>$keyword,
            'page'=>$page,
            'page_size'=>$page_size,
            'domain'=>request()->domain()
        ];
        $response = $client->get($this->tunkeUrl . '/v1/tm/crm_company_detail_search', [
            'query' => $data
        ]);
        $result = $response->getBody();
        $jsonData = \GuzzleHttp\json_decode($result, true);

        $resultData = isset($jsonData['data']['data'])?$jsonData['data']['data']:[];

        $total = $resultData['total'];
        $trademarkList = $resultData['data'];

        View::assign('trademarkList',$trademarkList);
        View::assign('total',$total);
        View::assign('page',$page);
        View::assign('totalPage',ceil($total/$page_size));
        View::assign('keyword',$keyword);

        return View::fetch('cbcrm@site/trademark/company_detail');
    }
}
