<?php


namespace app\cbcrm\services;


use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\trade\TrademarkLocalModel;
use app\cbcrm\model\trade\TrademarkOrderModel;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use PhpOffice\PhpWord\Element\TextRun;
use PhpOffice\PhpWord\TemplateProcessor;
use think\Exception;

class TrademarkService
{
    protected $tunkeUrl = 'https://www.tunke.cn';

    protected $tradeApiUrl = 'http://haoip.tunke.cn/api';

    use \liliuwei\think\Jump;
    /**
     * 商标注册 - 生成代理委托书
     * @param string $name 商标名称
     * @param array $tmApplicant 商标申请人
     * @param integer $wid 商户ID
     * */
    public function delegationFile($name,$tmApplicant,$wid)
    {
        try {
            $config = (new SettingModel())->getConfigGroup('trade',$wid);
            $templateFile = $_SERVER['DOCUMENT_ROOT'].'/app/cbcrm/site/images/tm_delegation_tmplate.docx';
            $templateProcessor = new TemplateProcessor($templateFile);// 加载模板文件
            $fontStyle = array('name'=>'宋体','size'=>12,'underline' => 'single');// 设置字体样式
            $APPLYER = new TextRun(); // 商标申请人
            $AGENT = new TextRun(); // 代理机构
            $TM = new TextRun(); // 商标名称
            $APPLYERADDRESS = new TextRun(); // 商标申请人地址
            $AGENTNAME = new TextRun(); // 代理机构联系人
            $AGENTTEL = new TextRun(); // 代理机构联系人电话
            $AGENTPOSTCODE = new TextRun(); // 代理机构联系邮编
            $DATE = new TextRun(); // 日期

            $APPLYER->addText($tmApplicant['name'],$fontStyle);
            $AGENT->addText($config['agent_company'],$fontStyle);
            $TM->addText($name,$fontStyle);
            $APPLYERADDRESS->addText($tmApplicant['address'],$fontStyle);
            $AGENTNAME->addText($config['agent_name'],$fontStyle);
            $AGENTTEL->addText($config['agent_tel'] ,$fontStyle);
            $AGENTPOSTCODE->addText($config['agent_post'],$fontStyle);
            unset($fontStyle['underline']);
            $DATE->addText(date('Y年 m 月 d 日'),$fontStyle);

            $templateProcessor->setComplexValue('APPLYER', $APPLYER);
            $templateProcessor->setComplexValue('AGENT', $AGENT);
            $templateProcessor->setComplexValue('TM', $TM);
            $templateProcessor->setComplexValue('APPLYERADDRESS', $APPLYERADDRESS);
            $templateProcessor->setComplexValue('AGENTNAME', $AGENTNAME);
            $templateProcessor->setComplexValue('AGENTTEL', $AGENTTEL);
            $templateProcessor->setComplexValue('AGENTPOSTCODE', $AGENTPOSTCODE);
            $templateProcessor->setComplexValue('DATE', $DATE);
            $fileName = $name.'【'.$tmApplicant['name'].'】.docx';
            $dir = $_SERVER['DOCUMENT_ROOT'].'/storage/upload/cbcrm/template';
            if (!file_exists($dir)) {
                @mkdir($dir, 0777, true);
            }
            $templateProcessor->saveAs($_SERVER['DOCUMENT_ROOT'].'/storage/upload/cbcrm/template/'.$fileName);
            $url = request()->domain().'/storage/upload/cbcrm/template/'.$fileName;
            return $url;
        } catch (\PhpOffice\PhpWord\Exception\Exception $e) {
            $this->error('文档生成失败');
        }
    }

    /**
     * 商标注册 - 生成商标图样
     * @param string $name 商标名称
     * @param string $font 字体
     * */
    public function createTmImage($name,$font)
    {

        $client = new Client();
        try {
            $response = $client->get($this->tunkeUrl . '/v1/crm/trademark/createImage', [
                'query' => [
                    'name' => $name,
                    'font' => $font
                ]
            ]);
            $result = $response->getBody();
            $jsonData = \GuzzleHttp\json_decode($result, true);
            if (isset($jsonData['result']) && !$jsonData['result']) {
                $this->error($jsonData['message']);
            } else {
                if ($jsonData['data']['data'] && isset($jsonData['data']['data']['url'])) {
                    $url = $this->tunkeUrl.$jsonData['data']['data']['url'];
                    return $url;
                }else{
                    return false;
                }
            }
        } catch (GuzzleException $e) {
            $this->error('商标图样生成失败：'.$e->getMessage());
        }
    }

    /**
     * 商标交易 - 获取商标云库商标详情
     * @param integer $id 云库商标ID
     * @param string $apiKey 商标交易密钥
     * */
    public function getCloudLibDetail($id,$apiKey)
    {
        $row = null;
        $client = new Client();
        try {
            $params = [
                'api_key' => $apiKey,
                'id' => $id,
            ];
            $response = $client->request('GET', $this->tradeApiUrl . '/trade/trademark/getCloudDetail', [
                'query' => $params
            ]);
            $result = json_decode($response->getBody(), true);
            if ($result['code'] != 1) {
                throw new Exception($result['msg']);
            }
            $row = $result['data'];

        } catch (GuzzleException|Exception $e) {
             $this->error($e->getMessage());
        }

        return $row;
    }


    /**
     * 商标交易 - 创建商标云库订单
     * @param integer $id 订单ID
     * @param string $apiKey 商标交易密钥
     * @param array $params 提交参数
     * */
    public function createCloudOrder($id,$apiKey,$params)
    {
        $client = new Client();
        try {
            $params['domain'] = \request()->domain();
            $response = $client->request('POST', $this->tradeApiUrl . '/trade/trademark_order/create', [
                'query' => ['api_key' => $apiKey],
                'json'=>$params
            ]);
            $result = json_decode($response->getBody(), true);
            if ($result['code'] != 1) {
                throw new Exception($result['msg']);
            }
            if (isset($result['data']['unique_id']) ) {
                // 生成唯一ID
                (new TrademarkOrderModel())->where('id',$id)->update([
                    'unique_id'=>$result['data']['unique_id'],
                    'update_time'=>date('Y-m-d H:i:s')
                ]);
            }

        } catch (GuzzleException|Exception $e) {
            $this->error($e->getMessage());
        }
    }


    /**
     * 商标交易 - 更新商标云库订单
     * @param string $apiKey 商标交易密钥
     * @param array $params 提交参数
     * */
    public function updateCloudOrder($apiKey,$params)
    {
        $client = new Client();
        try {
            $params['domain'] = \request()->domain();
            $response = $client->request('POST', $this->tradeApiUrl . '/trade/trademark_order/update', [
                'query' => ['api_key' => $apiKey],
                'json'=>$params
            ]);
            $result = json_decode($response->getBody(), true);
            if ($result['code'] != 1) {
                throw new Exception($result['msg']);
            }

        } catch (GuzzleException|Exception $e) {
            $this->error($e->getMessage());
        }
    }
}
